/*
 * SPDX-FileCopyrightText: 2015-2022 Espressif Systems (Shanghai) CO LTD
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#pragma once

#include <stdint.h>
#include "esp_err.h"

#ifdef __cplusplus
extern "C" {
#endif

    typedef struct {
        uint8_t : 5;
        uint8_t touch_type : 3;
        uint8_t tip : 1;
        uint8_t event_id : 2;
        uint8_t touch_id : 5;
        uint16_t x;
        uint16_t y;
        uint8_t pressure;
        uint16_t major_axis_length;
        uint8_t orientation;
    } __attribute__((packed)) touch_record_struct_t;

    typedef struct {
        uint16_t data_len;
        uint8_t report_id;
        uint16_t time_stamp;
        uint8_t : 2;
        uint8_t large_object : 1;
        uint8_t record_num : 5;
        uint8_t report_counter : 2;
        uint8_t : 3;
        uint8_t noise_efect : 3;
        touch_record_struct_t touch_record[0];
    } __attribute__((packed)) touch_report_struct_t;

    typedef struct {
        uint16_t length;        /*!< Always 14(0x000E) */
        uint8_t report_id;      /*!< Always 03h */
        uint16_t time_stamp;    /*!< Number in units of 100 us */
        uint8_t btn_val;        /*!< Only use bit[0..3] */
        uint16_t btn_signal[4];
    } __attribute__((packed)) button_record_struct_t;

    typedef struct {
        uint8_t  touch_state;
        uint16_t x;
        uint16_t y;
    } __attribute__((packed)) ew_touch_struct_t;

    typedef struct {
        uint8_t  btn_val;
        uint8_t  fingers;
        ew_touch_struct_t touch_record[0];
    } __attribute__((packed))ew_touch_report_struct_t;
    /**
     * @brief Init TT21100 touch panel
     *
     * @return
     *    - ESP_OK: Success
     *    - Others: Fail
     */
    esp_err_t tt21100_tp_init(void);

    /**
     * @brief Read packet from TT21100
     *
     * @return
     *    - ESP_OK: Success
     *    - Others: Fail
     */
    esp_err_t tt21100_tp_read(void);

    esp_err_t ew_tt21100_tp_read(ew_touch_report_struct_t* ew_touch);

    esp_err_t tt21100_read(uint8_t* data, size_t data_len);

    /**
     * @brief
     *
     * @param p_tp_num
     * @param p_x
     * @param p_y
     * @return esp_err_t
     */
    esp_err_t tt21100_get_touch_point(uint8_t* p_tp_num, uint16_t* p_x, uint16_t* p_y);

    /**
     * @brief
     *
     * @param p_btn_val
     * @param p_btn_signal
     * @return esp_err_t
     */
    esp_err_t tt21100_get_btn_val(uint8_t* p_btn_val, uint16_t* p_btn_signal);

    /**
     * @brief TT21100 will keep COMM_INT low until all data read.
     *        So if the INT line is low after read the packet, read again.
     *
     * @return true Data avaliable
     * @return false All data has been read
     */
    bool tt21100_data_avaliable(void);

#ifdef __cplusplus
}
#endif

