/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the board support package (BSP) of a dedicated
*   target.
*   This template provides access to some LEDs and buttons of the board.
*
*******************************************************************************/

#include "app_led.h"
#include <stdio.h>
#include "driver/gpio.h"
#include "esp32s3/rom/gpio.h"

#include "ew_bsp_inout.h"

#define BUTTON_GPIO           0

static TButtonCallback        ButtonCallback = NULL;

static void IRAM_ATTR gpio_isr_handler(void* arg)
{
  if ( ButtonCallback )
    ButtonCallback( !gpio_get_level( BUTTON_GPIO ));
}


/*******************************************************************************
* FUNCTION:
*   EwBspInOutInitButton
*
* DESCRIPTION:
*   Configures one hardware button of the board used for demo applications.
*
* ARGUMENTS:
*   aButtonCallback - The button callback.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspInOutInitButton( TButtonCallback aButtonCallback )
{
  gpio_config_t io_conf;

  /* configure gpio to detect falling and rising edge */
  io_conf.pull_down_en = 0;
  io_conf.pull_up_en   = 1;
  io_conf.intr_type    = GPIO_INTR_ANYEDGE;
  io_conf.pin_bit_mask = (1UL << BUTTON_GPIO);
  io_conf.mode         = GPIO_MODE_INPUT;
  gpio_config( &io_conf );

  /* add to the gpio isr service */
  gpio_isr_handler_add( BUTTON_GPIO, gpio_isr_handler, (void*)0 );

  /* store the callback function */
  ButtonCallback = aButtonCallback;
}


/*******************************************************************************
* FUNCTION:
*   EwBspInOutInitLed
*
* DESCRIPTION:
*   Configures one LED of the board used for demo applications.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspInOutInitLed( void )
{
  /* configure the LED gpio output */
  const board_res_desc_t *brd = bsp_board_get_description();
  app_pwm_led_init(brd->PMOD2->row1[1], brd->PMOD2->row1[2], brd->PMOD2->row1[3]);
}


/*******************************************************************************
* FUNCTION:
*   EwBspInOutLedOn
*
* DESCRIPTION:
*   Switch LED on (used for demo applications).
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspInOutLedOn( void )
{
  /* turn on the LED output */
  app_pwm_led_set_all(128, 0, 0);
}


/*******************************************************************************
* FUNCTION:
*   EwBspInOutLedOff
*
* DESCRIPTION:
*   Switch LED off (used for demo applications).
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspInOutLedOff( void )
{
  /* turn off the LED output */
	app_pwm_led_set_all(0, 0, 0);
}


/* psh */
